=begin
= ŷץ饰((-$Id: weather.rb,v 1.6 2004/08/10 18:28:42 zunda Exp $-))
ŷ򡢤ǽ˹˼¸줾
ξɽޤ

== ˡ
ΥեκǿǤϡ
((<URL:http://zunda.freeshell.org/d/plugin/weather.rb>))
ˤޤ

== Ȥ
=== 󥹥ȡˡ
Υեpluginǥ쥯ȥ˥ԡƤɤ
EUC-JPǤ

ˡtdiary.confԽ뤫WWW֥饦tDiary̤֤
ŷפǡŷǡ򤤤ƤURLꤷƤ
tdiary.confԽˤϡ@options['weather.url']ꤷƤ 
ξ򤷤ˤϡtDiary̤Ǥ꤬ͥ褵ޤ

㤨С NOAA National Weather ServiceѤˤϡ
((<URL:http://weather.noaa.gov/>))顢Select a country...ǹ̾
Go!ܥ򲡤˴¬Ǥλɽ줿ڡ
URL㤨С
  @options['weather.url'] = 'http://weather.noaa.gov/weather/current/RJTI.html'
Ƚ񤤤ƤǤإݡ((-ɤˤ-))ŷ
ϿޤѤ¤Ƥ礬ޤΤǡΤ
WWWڡʤ褦դƤ

ˡΥॾѲǽϡΥॾ
@options['weather.tz']ĶѿTZꤷƤȤ򤪴ᤷ 
ޤˤäơۤ⡢ŷǡΥॾ 
ŷɽ³뤳ȤǤޤtdiary.confꤹϡ㤨
ɸξϡ
  @options['weather.tz'] = 'Japan'
ꤷƤ

ǡ٤ˡꤷURLŷǡơ
ɽ褦ˤʤϤǤŷϡ
  <div class="weather"><span class="weather">hh:mm<a href="URL">ŷ()</a></span></div>
ȤǤ줾ξɽޤɬפʤСCSSԽ
Ƥ
  div.weather {
    text-align: right;
    font-size: 75%;
  }
ʤɤȤƤФǤ礦

˻ѤƤWWWС饵Сθ¤WWWڡαǤ
ɬפޤĶѿTZѹ礬ޤΤǡsecure⡼ɤǤ
Ȥޤmod_rubyǤưϺΤȤǧƤޤ

ǥեȤǤϡü줿ˤŷɽʤ褦ˤʤ
ƤޤӤǤŷɽˤϡ̤ꤹ뤫
tdiary.conf
  @options['weather.show_mobile'] = true
ꤷƤ

=== ¸ŷǡˤĤ
ŷǡϡ
* 񤤤Ƥդȸߤդפ
* ŷǡޤƤʤμ˥顼ä
ˡޤ

ŷǡϡ@options['weather.dir']ꤷǥ쥯ȥ꤫
@cache_path/weather/ ǥ쥯ȥʲˡǯ/ǯ.weather Ȥե 
̾¸ޤֶڤΥƥȥեǤΤɬפ˱Խ 
뤳ȤǤޤ֤οѤƤޤʤ褦˵ĤԽƤ
եޥåȤξܺ٤ϡWeather.to_s᥽åɤ򻲾ȤƤ

ŷǡˤϡǡμ郎ϿƤޤޤǡμ
줿ŷι郎ϿƤ뤳Ȥ⤢ޤλ
ϡɸ(UNIX)ľƵϿƤơɽ˸
ľƤޤΤᡢŷϿΥॾȡŷɽ
Υॾ󤬰ۤʤäƤޤȡ㤨īŷäΤͼ
ŷȤɽƤޤȤˤʤޤɤˤϡ㤨С
  @options['weather.tz'] = 'Japan'
Ȥץꤷơǡ˥ॾϿ褦ˤƤ
tdiary.confʤɤǡ
  ENV['TZ'] = 'Japan'
ʤɤȤƴĶѿTZꤹ뤳ȤǤƱͤθ̤ޤĶѿ
ꤷϡtDiaryΤư˱ƶޤΤαդƤ

ʤ1.1.2.19ΥСweather.rbǤϥॾξ
ŷǡ˵ϿƤޤ󡣤ǤɬפʤСեԽ
ơॾɲäƤϿեϡǥեȤǤϡ
  .../cache/weather/2003/200301.weather
ʤɤˤޤURLμοUNIXǤΤǡ³ơ
ĤȡJapanʤɥॾ򼨤ʸϤƤǡ
˥顼ʤСθ売ĤΥ֤³ơŷΥǡϿ
ƤϤǤ

=== ץ
==== ɬ꤬ɬפʹ
: @options['weather.url']
  ŷǡWWWڡURL
    @options['weather.url'] = 'http://weather.noaa.gov/weather/current/RJTI.html'
  ʤɡѤ¤Ƥ礬ޤΤǡΤ褦WWW
  ڡʤ褦դƤ֥饦ꤷ
  Ϥ餬ͥ褵ޤ

==== ꤷʤƤ⤤
: @options['weather.show_mobile'] = false
  trueξϡüΥξˡi_html_string
  CHTMLɽޤfalseξϡüΥξˤŷ
  ɽޤ󡣥֥饦ꤷϤ餬ͥ褵ޤ
  
: @options['weather.tz']
  ǡΥॾ󡣥ޥɥ饤㤨С
    TZ=Japan date
  ¹Ԥ郎ɽʸꤷƤLinuxǤϡ
  /usr/share/zoneinfoʲΥե̾ꤹФϤǤ֥饦
  ꤷϤ餬ͥ褵ޤΥץ󤬻ꤵƤ
  ʤ硢ĶѿTZꤵƤФͤѤޤǤʤ
  ХॾϵϿޤ
   
  ŷǡ˥ॾ󤬵ϿƤʤϡ⤷Υ
  ѹ줿˰㤦ɽ뤳Ȥˤʤޤ
  
  դȽʤɡŷǡεϿʳλδˤϡΤΥ
  Ѥޤ

: @options['weather.oldest'] = 21600
  줿ǡΥץ()Ťˤϡŷμ顼
  ˤʤꡢιǺƤӥǡ褦ȤޤǥեȤ6
  (21600)ǤΥץnilꤵƤˤϡɤ
  ˸ŤǡǤޤ

: @options['weather.show_error']
  ǡ˥顼äˤɽˤtrue
  ޤǥեȤǤɽޤ

: @options['weather.dir']
  ǡ¸ꡣǥեȤϰʲ̤ꡣ
    "#{@cache_path}/weather/"
  βˡǯ/ǯ.weather Ȥե뤬ޤ
  @data_pathƱˤȡΥǡƱǥ쥯ȥŷΥǡ
  ¸Ǥ뤫⤷ޤ

: @options['weather.items']
  WWWڡܡǥեȤϡ
  parse_html̾򥭡Ͽ̾ͤȤϥåǤ
  www.nws.noaa.govΥեޥåȤ˹碌ơ¿ñ̤ưˤѤ
  褦ˤƤޤѹˤϡparse_html᥽åɤ
  ɬפ뤫⤷ޤ

: @options['weather.header']
  HTTPꥯȥإåɲäܤΥϥå
    @options['weather.header'] = {'Accept-language' => 'ja'}
  ʤɡ((-Accept-languageˤäƼ٤륵Ȥ⤢ޤ-))
  ǥեȤǤɲäΥإåޤ

== ŷˤĤ
NWSΥǡϱѸǤΤǡŬܸľƤϤ褦ˤ
ƤޤϡWeatherTranslator⥸塼ˤäƤơѴɽϡ
Weather饹ˡWords_jaȤȤͿƤޤ

äϤޤޤʬǤϤʤȻפޤΤʤñϱѸΤޤɽ
ΤǡWords_jaŬɲäƤ
((<URL:http://tdiary-users.sourceforge.jp/cgi-bin/wiki.cgi?weather%2Erb>))
˽񤤤ƤȡΤ۸ɲä뤫⤷ޤ

== ٤
ŷǡ乥ߤ˹ơʲΥ᥽åɤѹ뤳Ȥǡ 
꤬Ǥޤ

=== ɽ˴ؤ
ǥեȤǤϡŷǡϡHTML_STARTHTML_ENDꤵƤʸ 
ǰϤޤޤdivspanΥ饹ѹˤϡѹ 
ǽʬǤʾѹɬפʾϰʲѹƤ

: Weather.html_string
  @data[item]򻲾ȤơŷɽHTMLҤäƤ

: Weather.error_html_string
  ǡ顼äˡ@error򻲾Ȥƥ顼ɽHTML
  ҤäƤ

üακݤˤϡ
  @options['weather.show_mobile'] = true
ξˤϡ嵭ˡ줾I_HTML_STARTI_HTML_END
Weather.i_html_stringȤޤ顼ɽϤǤޤ

=== ŷǡμ˴ؤ
: Weather.parse_html( html, items )
  ((|html|))ʸϤơ((|items|))ϥå˽ä@data[item]
  Ƥ((|items|))ˤ@optins['weather.items']ޤ
  Weather_default_itemsޤ֤ͤѤޤ󡣥ơ֥ 
  Ѥŷ󸻤ʤСΥ᥽åɤ򤢤ޤ¤ʤǻȤ뤫 
  ޤ

== ޤ٤
* ŷ˱ɽ -ɤ

== ռ
ŷץ饰Υǥ󶡤Ƥähsbt󡢼Υ
Ȥ󶡤Ƥäzoe˴դޤޤNOAAξ󶡤
äkotak˴դޤ

The author appreciates National Weather Service
((<URL:http://weather.noaa.gov/>)) making such valuable data available
in public domain as described in ((<URL:http://www.noaa.gov/wx.html>)).

== Copyright
Copyright 2003 zunda <zunda at freeshell.org>

Permission is granted for use, copying, modification, distribution,
and distribution of modified versions of this work under the terms
of GPL version 2 or later.
=end

=begin
== Instance variables
=end
@weather_plugin_name = 'ŷ'

=begin
== Classes and methods
=== WeatherTranslator module
We want Japanese displayed in a diary written in Japanese.

--- Weather::Words_ja
    Array of arrays of a Regexp and a Statement to be executed.
    WeatherTranslator::S.tr accepts this kind of hash to translate a
    given string.
=end

class Weather
	Words_ja = [
		[%r[\A(.*)/(.*)], '"#{S.new( $1 ).translate( table )}/#{S.new( $2 ).translate( table )}"'],
		[%r[\s*\b(greater|more) than (-?[\d.]+\s*\S*)\s*]i, '"#{S.new( $2 ).translate( table )}ʾ"'],
		[%r[^(.*?) with (.*)$]i, '"#{S.new( $2 ).translate( table )}#{S.new( $1 ).translate( table )}"'],
		[%r[^(.*?) during the past hours?$]i, '"ľޤ#{S.new( $1 ).translate( table )}"'],
		#[%r[\s*\b([\w\s]+?) in the vicinity]i, '"դ#{S.new( $1).translate( table )}"'],
		[%r[\s*\bin the vicinity\b\s*]i, '""'],
		# ... in the vicinity̵뤵褦ˤʤäƤޤ줬ߤϡ
		# ΥȥȤƤԤΥȤ򳰤Ƥ
		[%r[\s*\bpatches of\b\s*]i, '""'],
		[%r[\s*\bdirection variable\b\s*]i, '""'],
		[%r[\s*(-?[\d.]+)\s*\(?F\)?], '"ڻ#{$1}"'],
		[%r[\s*\bmile(\(?s\)?)?\s*]i, '"ޥ"'],
		[%r[\s*\b(mostly |partly )clear\b\s*]i, '""'],
		[%r[\s*\bclear\b\s*]i, '""'],
		[%r[\s*\b(mostly |partly )?cloudy\b\s*]i, '""'],
		[%r[\s*\bovercast\b\s*]i, '""'],
		[%r[\s*\blight snow showers?\b\s*]i, '"ˤ狼"'],
		[%r[\s*\blight snow\b\s*]i, '""'],
		[%r[\s*\blight drizzle\b\s*]i, '""'],
		[%r[\s*\blight rain showers?\b\s*]i, '"夤ˤ狼"'],
		[%r[\s*\bheavy rain showers?\b\s*]i, '"ˤ狼"'],
		[%r[\s*\bheavy rain\b\s*]i, '"뱫"'],
		[%r[\s*\b(rain )?showers?\b\s*]i, '"ˤ狼"'],
		[%r[\s*\bdrizzle\b\s*]i, '"̤"'],
		[%r[\s*\blight rain\b\s*]i, '"̸"'],
		[%r[\s*\brain\b\s*]i, '""'],
		[%r[\s*\bmist\b\s*]i, '""'],
		[%r[\s*\bhaze\b\s*]i, '""'],
		[%r[\s*\b(partial )?(freezing )?fog\b\s*]i, '"̸"'],
		[%r[\s*\bsnow\b\s*]i, '""'],
		[%r[\s*\bthunder( storm)?\b\s*]i, '""'],
		[%r[\s*\blightning\b\s*]i, '""'],
		[%r[\s*\bsand\b\s*]i, '""'],
		[%r[\s*\bcumulonimbus clouds\b\s*]i, '""'],
		[%r[\s*\bcumulus clouds\b\s*]i, '"ѱ"'],
		[%r[\s*\btowering\b\s*]i, '""'],
		[%r[\s*\bobserved\b\s*]i, '""'],
		[%r[\s*\bC\b\s*], '""'],
	].freeze
end

=begin
=== Weather class
Weather of a date.
--- Weather.html_string
--- Weather.error_html_string
      Returns an HTML fragment showing data or error, called from
      Weather.to_html.

--- Weather.i_html_string
      Returns a CHTML fragment to be shown on a mobile browser.
=end
class Weather

	def error_html_string
		%Q|#{HTML_START}ŷ顼:<a href="#{@url}">#{CGI::escapeHTML( @error )}</a>#{HTML_END}|
	end

	# edit this method to define how you show the weather
	def html_string
		r = "#{HTML_START}"

		# time stamp
		if @tz then
			tzbak = ENV['TZ']
			ENV['TZ'] = @tz	# this is not thread safe...
		end
		if @data['timestamp'] then
			r << Time::at( @data['timestamp'].to_i ).strftime( '%H:%M' ).sub( /^0/, '' )
		else
			r << Time::at( @time.to_i ).strftime( '%H:%M' ).sub( /^0/, '' )
		end
		r << ''
		if @tz then
			ENV['TZ'] = tzbak
		end

		# weather
		r << %Q|<a href="#{@url}">|
		if @data['weather'] then
			r << CGI::escapeHTML( WeatherTranslator::S.new( @data['weather']).translate( Words_ja ).compact )
		elsif @data['condition'] then
			r << CGI::escapeHTML( WeatherTranslator::S.new( @data['condition']).translate( Words_ja ).compact )
		end

		# temperature
		if @data['temperature(C)'] and t = @data['temperature(C)'].scan(/-?[\d.]+/)[-1] then
			r << %Q| #{sprintf( '%.0f', t )}|
		end

		r << "</a>#{HTML_END}\n"
	end

	# edit this method to define how you show the weather for a mobile agent
	def i_html_string
		r = ''

		# weather
		if @data['weather'] then
			r << "#{I_HTML_START}"
			r << %Q|<A HREF="#{@url}">|
			r << CGI::escapeHTML( WeatherTranslator::S.new( @data['weather']).translate( Words_ja ).compact )
			r << "</A>#{I_HTML_END}\n"
		elsif @data['condition'] then
			r << "#{I_HTML_START}"
			r << %Q|<A HREF="#{@url}">|
			r << CGI::escapeHTML( WeatherTranslator::S.new( @data['condition']).translate( Words_ja ).compact )
			r << "</A>#{I_HTML_END}\n"
		end

	end
end

# www configuration interface
def weather_configure_html( conf )
	<<-HTML
	<h3 class="subtitle">ŷץ饰</h3>
	<p>ŷ򡢤ǽ˹˼¸
		줾ξɽޤ</p>
	<h4>ŷǡ</h4>
	<p>ŷ㤨NOAA National Weather ServiceѤˤϡ
		<a href="http://weather.noaa.gov/">NOAA National Weather Service</a>
		顢Select a country...ǹ̾Go!ܥ򲡤
		˴¬Ǥ
		ơλɽ줿ڡURL򡢰ʲ˵Ƥ</p>
	<p><input name="weather.url" value="#{conf['weather.url']}" size="60"></p>
	<p>ΥॾѲǽϡ
		ΥॾϿƤȤ򤪴ᤷޤ
		ˤäơۤ⡢
		ŷǡΥॾŷɽ³뤳ȤǤޤ</p>
	<p>ॾϿˤϡ㤨ɸξˤϡ
		tdiary.rbƱǥ쥯ȥˤtdiary.confˡ
		ENV['TZ'] = 'Japan'ʤɤȽ­
		ʲˡJapanȵƤ</p>
	<p><input name="weather.tz" value="#{conf['weather.tz']}"></p>
	<h4>äؤɽ</h4>
	<p>Ǥ</p>
	<p><select name="weather.show_mobile">
		<option value="true"#{' selected'if conf['weather.show_mobile']}>
		äˤ⺣ŷɽ
		<option value="false"#{' selected'unless conf['weather.show_mobile']}>
		äˤϺŷɽʤ
	</select></p>
	<h4>󥸥ʤɤΥܥåȤؤɽ</h4>
	<p>Ǥ</p>
	<p><select name="weather.show_robot">
		<option value="true"#{' selected'if conf['weather.show_robot']}>
		󥸥ʤɤΥܥåȤˤ⺣ŷΤ餻
		<option value="false"#{' selected'unless conf['weather.show_robot']}>
		󥸥ʤɤΥܥåȤˤϺŷΤ餻ʤ
	</select></p>
	<h4>¾</h4>
	<p>¾ˤ⤤Ĥtdiary.confǤܤޤ
		ܤϡץ饰Υե(weather.rb)</p>
	HTML
end

